/*
    PPDocument_LayerOperationTarget.m

    Copyright 2013-2016 Josh Freeman
    http://www.twilightedge.com

    This file is part of PikoPixel for Mac OS X and GNUstep.
    PikoPixel is a graphical application for drawing & editing pixel-art images.

    PikoPixel is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License version 3,
    as published by the Free Software Foundation.

    PikoPixel is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#import "PPDocument.h"

#import "PPDocument_Notifications.h"
#import "PPDocumentLayer.h"


@implementation PPDocument (LayerOperationTarget)

- (void) setLayerOperationTarget: (PPLayerOperationTarget) operationTarget
{
    if (!PPLayerOperationTarget_IsValid(operationTarget)
        || _isPerformingInteractiveMove)
    {
        return;
    }

    _layerOperationTarget = operationTarget;

    [self postNotification_ChangedLayerOperationTarget];
}

- (PPLayerOperationTarget) layerOperationTarget
{
    return _layerOperationTarget;
}

- (NSBitmapImageRep *) sourceBitmapForLayerOperationTarget:
                                                    (PPLayerOperationTarget) operationTarget
{
    switch (operationTarget)
    {
        case kPPLayerOperationTarget_DrawingLayerOnly:
        {
            return _drawingLayerBitmap;
        }
        break;

        case kPPLayerOperationTarget_VisibleLayers:
        default:
        {
            return _mergedVisibleLayersBitmap;
        }
        break;
    }
}

- (void) setupTargetLayerIndexesForOperationTarget: (PPLayerOperationTarget) operationTarget
{
    switch (operationTarget)
    {
        case kPPLayerOperationTarget_DrawingLayerOnly:
        {
            if (_indexOfDrawingLayer >= 0)
            {
                _targetLayerIndexes[0] = _indexOfDrawingLayer;
                _numTargetLayerIndexes = 1;
            }
            else
            {
                _numTargetLayerIndexes = 0;
            }
        }
        break;

        case kPPLayerOperationTarget_VisibleLayers:
        {
            PPDocumentLayer *layer;
            int i;

            _numTargetLayerIndexes = 0;

            for (i=0; i<_numLayers; i++)
            {
                layer = [_layers objectAtIndex: i];

                if ([layer isEnabled])
                {
                    _targetLayerIndexes[_numTargetLayerIndexes++] = i;
                }
            }
        }
        break;

        case kPPLayerOperationTarget_Canvas:
        {
            int i;

            for (i=0; i<_numLayers; i++)
            {
                _targetLayerIndexes[i] = i;
            }

            _numTargetLayerIndexes = _numLayers;
        }
        break;

        default:
        {
            _numTargetLayerIndexes = 0;
        }
        break;
    }
}

- (bool) hasEnabledLayerForOperationTarget: (PPLayerOperationTarget) operationTarget
{
    switch (operationTarget)
    {
        case kPPLayerOperationTarget_DrawingLayerOnly:
        {
            if ((_indexOfDrawingLayer >= 0) && [_drawingLayer isEnabled])
            {
                return YES;
            }
        }
        break;

        case kPPLayerOperationTarget_VisibleLayers:
        case kPPLayerOperationTarget_Canvas:
        {
            PPDocumentLayer *layer;
            int i;

            for (i=0; i<_numLayers; i++)
            {
                layer = [_layers objectAtIndex: i];

                if ([layer isEnabled])
                {
                    return YES;
                }
            }
        }
        break;

        default:
        break;
    }

    return NO;
}

- (NSString *) nameOfLayerOperationTarget: (PPLayerOperationTarget) operationTarget
{
    NSString *targetName;

    switch (operationTarget)
    {
        case kPPLayerOperationTarget_DrawingLayerOnly:
        {
            targetName = (_hasSelection) ? @"Drawing Layer Selection" : @"Drawing Layer";
        }
        break;

        case kPPLayerOperationTarget_VisibleLayers:
        {
            targetName = (_hasSelection) ? @"Enabled Layers Selection" : @"Enabled Layers";
        }
        break;

        case kPPLayerOperationTarget_Canvas:
        {
            targetName = @"Canvas";
        }
        break;

        default:
        {
            targetName = @"UNKNOWN TARGET";
        }
        break;
    }

    return targetName;
}

@end
